<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://juaala.com/
 * @since      1.0.0
 *
 * @package    Rsonline_Juaala
 * @subpackage Rsonline_Juaala/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Rsonline_Juaala
 * @subpackage Rsonline_Juaala/includes
 * @author     Juaala <info@juaala.com>
 */
class Rsonline_Juaala {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Rsonline_Juaala_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The unique identifier the file of this plugin.
	 *
	 * @since    1.0.3
	 * @access   protected
	 * @var      string    $file_plugin    The string used to uniquely identify file this plugin.
	 */
	protected $file_plugin;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct($file_plugin) {
		if ( defined( 'RSONLINE_JUAALA_VERSION' ) ) {
			$this->version = RSONLINE_JUAALA_VERSION;
		} else {
			$this->version = '1.0.0';
		}
		$this->plugin_name = 'rsonline-juaala';
		$this->file_plugin = $file_plugin;

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();

	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Rsonline_Juaala_Loader. Orchestrates the hooks of the plugin.
	 * - Rsonline_Juaala_i18n. Defines internationalization functionality.
	 * - Rsonline_Juaala_Admin. Defines all hooks for the admin area.
	 * - Rsonline_Juaala_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-rsonline-juaala-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-rsonline-juaala-i18n.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-rsonline-juaala-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-rsonline-juaala-public.php';

		/**
		 * Load scss lib generation
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/libs/scssphp/scssphp-1.10.3/scss.inc.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/libs/scssphp/server/src/Server.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/libs/scssphp/server/src/ServerException.php';


		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-rsonline-juaala-api-controller.php';

		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-rsonline-juaala-template-controller.php';

		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-rsonline-juaala-update-checker.php';


		$this->loader = new Rsonline_Juaala_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Rsonline_Juaala_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new Rsonline_Juaala_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {

		$plugin_admin = new Rsonline_Juaala_Admin( $this->get_plugin_name(), $this->get_file_plugin(), $this->get_version() );

		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );
		$this->loader->add_action( 'admin_enqueue_scripts',  $plugin_admin, 'codemirror_enqueue_scripts');

		// Hook para cargar la documentación
		$this->loader->add_action('wp_ajax_get_doc', $plugin_admin, 'get_doc', 10, 2);

		//definir acciones del admin
		$this->loader->add_action( 'admin_menu',  $plugin_admin, 'juaala_rso_add_admin_menu');
    $this->loader->add_action( 'admin_init', $plugin_admin, 'juaala_rso_admin_init');

		//ejemplo de: https://pluginrepublic.com/wordpress-plugin-update-hook-upgrader_process_complete/
		$this->loader->add_action( 'upgrader_process_complete', $plugin_admin, 'juaala_wp_upe_upgrade_completed', 10, 2);

		$this->loader->add_action( 'juaala_rso_pdf_geretaror_job', $plugin_admin, 'juaala_rso_pdf_geretaror_job', 10, 18);

		$this->loader->add_filter( 'pre_update_option_rojuaala_primary_color', $plugin_admin, 'rojuaala_primary_color_save', 10, 2 );
	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {

		$plugin_public = new Rsonline_Juaala_Public( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );

		 /**
     * Register shortcode via loader
     *
     * Use: [short-code-name args]
     *
     * @link https://github.com/DevinVinson/WordPress-Plugin-Boilerplate/issues/262
     */
    $this->loader->add_shortcode( "juaala_rso_search", $plugin_public, "juaala_rso_search_shortcode");
		$this->loader->add_shortcode( "juaala_rso_properties", $plugin_public, "juaala_rso_properties_shortcode");
		$this->loader->add_shortcode( "juaala_rso_properties_by_refs", $plugin_public, "juaala_rso_properties_by_refs_shortcode");
		$this->loader->add_shortcode( "juaala_rso_property", $plugin_public, "juaala_rso_property_shortcode");
		$this->loader->add_shortcode( "juaala_rso_shortlist", $plugin_public, "juaala_rso_shortlist_shortcode");
		$this->loader->add_shortcode( "juaala_rso_screen", $plugin_public, "juaala_rso_screen_properties");
		$this->loader->add_shortcode( "juaala_rso_search_map", $plugin_public, "juaala_rso_search_shortcode_map");
		$this->loader->add_shortcode( "juaala_rso_property_hero", $plugin_public, "juaala_rso_property_hero_shortcode");
		$this->loader->add_shortcode( "juaala_rso_compare", $plugin_public, "juaala_rso_compare_shortcode");



		$this->loader->add_filter( 'query_vars', $plugin_public, 'juaala_rso_addnew_query_vars', 10, 1);

		$this->loader->add_action('admin_post_custom_form_submit', $plugin_public, 'juaala_rso_custom_form_submit');
		$this->loader->add_action( 'admin_post_nopriv_custom_form_submit', $plugin_public, 'juaala_rso_custom_form_submit');
		$this->loader->add_action('init', $plugin_public, 'juaala_rso_custom_rewrite_basic');

		$this->loader->add_action( 'the_post',  $plugin_public , 'juaala_rso_details_prorperty_post_action',999);
		$this->loader->add_action( 'pre_get_posts', $plugin_public, 'juaala_rso_set_property_details_page',999 );

		$this->loader->add_filter( 'document_title_parts', $plugin_public ,'custom_title_property_details',999,1 );

		$this->loader->add_filter( 'shortcode_atts_wpcf7', $plugin_public ,'juaala_rso_custom_shortcode_atts_wpcf7_filter', 999, 3 );
		$this->loader->add_action( 'wpcf7_before_send_mail', $plugin_public, 'juaala_rso_wpcf7_parse_property_field', 10, 1 );
		$this->loader->add_filter('property_email_info', $plugin_public, 'juaala_rso_property_email_info');
		$this->loader->add_filter('wpforms_smart_tags',$plugin_public, 'juaala_wpf_dev_register_smarttag', 10, 1 );
		$this->loader->add_action('wpforms_process_before', $plugin_public, 'juaala_rso_wpf_dev_process_before', 10, 2 );
		$this->loader->add_filter('wpforms_smart_tag_process', $plugin_public, 'juaala_wpf_dev_process_smarttag', 10, 2 );

		$this->loader->add_action( 'template_redirect', $plugin_public, 'redirect_direct_access');

		// Hook para el shorlist de propiedades
		$this->loader->	add_action('wp_ajax_nopriv_get_shortlist_properties', $plugin_public, 'get_shortlist_properties');

		// Hook para usuarios logados shorlist propiedades
		$this->loader->add_action('wp_ajax_get_shortlist_properties', $plugin_public, 'get_shortlist_properties');

		// Hook para añadir una propiedad al shortlist a través de cookies
		$this->loader->add_action('wp_ajax_nopriv_set_property_shortlist', $plugin_public, 'set_property_shortlist');

		// Hook para añadir una propiedad al shortlist a través de cookies
		$this->loader->add_action('wp_ajax_set_property_shortlist', $plugin_public, 'set_property_shortlist');

		// Hook para eliminar una propiedad al shortlist a través de cookies
		$this->loader->add_action('wp_ajax_nopriv_remove_property_shortlist', $plugin_public ,'remove_property_shortlist');

		// Hook para eliminar una propiedad al shortlist a través de cookies
		$this->loader->add_action('wp_ajax_remove_property_shortlist', $plugin_public, 'remove_property_shortlist');


		// Hook para añadir una propiedad al compare a través de cookies
		$this->loader->add_action('wp_ajax_nopriv_set_property_compare', $plugin_public, 'set_property_compare');

		// Hook para añadir una propiedad al compare a través de cookies
		$this->loader->add_action('wp_ajax_set_property_compare', $plugin_public, 'set_property_compare');

		// Hook para eliminar una propiedad al compare a través de cookies
		$this->loader->add_action('wp_ajax_nopriv_remove_property_compare', $plugin_public ,'remove_property_compare');

		// Hook para eliminar una propiedad al compare a través de cookies
		$this->loader->add_action('wp_ajax_remove_property_compare', $plugin_public, 'remove_property_compare');


		$this->loader->add_action('get_footer', $plugin_public ,'write_shorlist',11,0);
		$this->loader->add_action('get_footer', $plugin_public ,'write_compare_list',11,0);

		// Hook para usuarios no logueados
		$this->loader->add_action('wp_ajax_nopriv_get_number_properties', $plugin_public, 'get_number_properties');

		// Hook para usuarios logueados
		$this->loader->add_action('wp_ajax_get_number_properties',$plugin_public ,'get_number_properties');

		// Hook para usuarios no logueados
		$this->loader->add_action('wp_ajax_nopriv_get_map_property', $plugin_public, 'get_map_property');

		// Hook para usuarios logueados
		$this->loader->add_action('wp_ajax_get_map_property', $plugin_public, 'get_map_property');

		// Hook para el shorlist de propiedades
		$this->loader->add_action('wp_ajax_nopriv_get_features_properties', $plugin_public, 'get_features_properties');

		// Hook para usuarios logueados shorlist propiedades
		$this->loader->add_action('wp_ajax_get_features_properties', $plugin_public, 'get_features_properties');

		// Hook para las propiedades relaccionadas
		$this->loader->add_action('wp_ajax_nopriv_get_releated_properties', $plugin_public, 'get_releated_properties',10, 4);

		// Hook para usuarios logueados propiedades relaccionadas
		$this->loader->add_action('wp_ajax_get_releated_properties', $plugin_public, 'get_releated_properties', 10, 4);


		// Hook para el pdf de propiedad
		add_action('wp_ajax_nopriv_get_property_pdf', $plugin_public, 'get_property_pdf');

		// Hook para usuarios logueados pdf propiedad
		add_action('wp_ajax_get_property_pdf', $plugin_public, 'get_property_pdf');

	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

		/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.3
	 * @return    string    The name of the plugin.
	 */
	public function get_file_plugin() {
		return $this->file_plugin;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Rsonline_Juaala_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

}
